/**
 * Solution to Problem 2 (Node.js Web Service) in 19su Exam 2
 */

"use strict";
const express = require("express");
const fs = require("fs");
const util = require("util");
const path = require("path");
const readdir = util.promisify(fs.readdir);
const readFile = util.promisify(fs.readFile);
const glob = require("glob");
const globPromise = util.promisify(glob);

const app = express();
app.use(express.static("public"));

const SERVER_ERROR_MSG = "Something went wrong on the server, please try again later.";

// Part A: Endpoint 1
app.get("/courses", async (req, res) => {
  res.type("text");
  try {
    let files = await readdir("data");
    let result = "";
    for (let i = 0; i < files.length; i++) {
      let courseName = path.basename(files[i]).replace(".txt", "");
      result += courseName + "\n";
    }
    res.send(result.trim());
  } catch (err) {
    res.status(500).send(SERVER_ERROR_MSG);
  }
});

// Part B: Endpoint 2
app.get("/books/:course", async (req, res) => {
  let classname = req.params.course;
  try {
    let data = await readFile("data/" + classname + ".txt", "utf-8");
    let result = {};
    let lines = data.split("\n");
    result.name = lines[0];
    let minPrice;
    let minSeller;
    for (let i = 1; i < lines.length; i++) {
      let lineParts = lines[i].split(":");
      let seller = lineParts[0];
      let price = parseFloat(lineParts[1]);
      if (!minPrice || price < minPrice) {
        minPrice = price;
        minSeller = seller;
      }
    }
    result.price = minPrice;
    result.seller = minSeller;
    res.json(result);
  } catch (err) {
    if (err.name === "ENOENT") {
      res.status(400).send("No data found for course.");
    } else {
      res.status(500).send(SERVER_ERROR_MSG);
    }
  }
});

const PORT = process.env.PORT || 8000;
app.listen(PORT);
