/**
 * Solution to Problem 3 (Client-side JS with AJAX) in 19su Exam 2
 */
"use strict";
(function() {
  window.addEventListener("load", init);

  function init() {
    fetch("/courses")
      .then(checkStatus)
      .then(resp => resp.text())
      .then(populateClasses)
      .catch(displayError);
  }

  function populateClasses(resp) {
    resp = resp.trim().split("\n");
    let courses = id("courses");
    for (let courseName of resp) {
      let btn = gen("button");
      btn.textContent = courseName;
      btn.addEventListener("click", getCheapestBook);
      courses.appendChild(btn);
    }
  }

  function getCheapestBook() {
    let classname = this.textContent;
    fetch("/books/" + classname)
      .then(checkStatus)
      .then(resp => resp.json())
      .then(populateInfo)
      .catch(displayError);
  }

  function populateInfo(resp) {
    id("book-info").classList.remove("hidden");
    id("book-name").textContent = resp.name;
    id("seller").textContent = resp.seller;
    id("price").textContent = resp.price;
  }

  function displayError() {
    id("book-info").classList.add("hidden");
    id("courses").classList.add("hidden");
    id("error-info").classList.remove("hidden");
  }

  /** ------------------------------ Helper Functions  ------------------------------ */
  /**
   * Returns the element that has the ID attribute with the specified value.
   * @param {string} idName - element ID
   * @returns {object} DOM object associated with id.
   */
  function id(idName) {
    return document.getElementById(idName);
  }

  /**
   * Returns the first element that matches the given CSS selector.
   * @param {string} selector - CSS query selector.
   * @returns {object} The first DOM object matching the query.
   */
  function qs(selector) {
    return document.querySelector(selector);
  }

  /**
   * Returns the array of elements that match the given CSS selector.
   * @param {string} selector - CSS query selector
   * @returns {object[]} array of DOM objects matching the query.
   */
  function qsa(selector) {
    return document.querySelectorAll(selector);
  }

  /**
   * Returns a new element with the given tag name.
   * @param {string} tagName - HTML tag name for new DOM element.
   * @returns {object} New DOM object for given HTML tag.
   */
  function gen(tagName) {
    return document.createElement(tagName);
  }

  function checkStatus(resp) {
    if (!resp.ok) {
      throw Error("Error: " + resp.statusText);
    }
    return resp;
  }

})();
